@@ -36,6 +36,7 @@
 #' @importFrom GEOquery getGEOSuppFiles
 #' @importFrom dplyr %>%
 #' @importFrom utils URLdecode
+#' @importFrom httr HEAD
 #' @noRd
 .downloadSamples <- function(sampleIDs, protocol, destDir) {
 
@@ -53,7 +54,38 @@
 
             if (file.exists(file.path(destDir, paste0(id, ".CEL.gz")))) next()
 
-            downloadedFiles <- getGEOSuppFiles(id, baseDir = destDir, makeDirectory = FALSE) %>% rownames()
+            filesURL <- getGEOSuppFiles(id, baseDir = destDir, makeDirectory = FALSE, fetch_files = FALSE)
+
+            actualInfo <- lapply(1:nrow(filesURL), function(curRow){
+                actualFileLength <- httr::HEAD(filesURL[curRow, 2])$headers$`content-length`
+                data.frame(
+                    fname = filesURL[curRow, 1],
+                    fsize = actualFileLength,
+                    stringsAsFactors = FALSE
+                )
+            }) %>% do.call(what = rbind)
+
+            downloadedInfo <- getGEOSuppFiles(id, baseDir = destDir, makeDirectory = FALSE)
+            downloadedFiles <- downloadedInfo %>% rownames()
+
+            isDeletedFlag <- FALSE
+            for(i in 1:2){
+                 lapply(nrow(downloadedInfo), function(curRow){
+                    if(downloadedInfo[curRow, "size"] != actualInfo[curRow, "fsize"]){
+                        file.remove(downloadedFiles[curRow])
+                        isDeletedFlag <- TRUE
+                    }
+                })
+
+                if(i == 2 & isDeletedFlag == TRUE){
+                    stop(paste0("There is an error in downloading sample ", id))
+                }else if(isDeletedFlag){
+                    downloadedInfo <- getGEOSuppFiles(id, baseDir = destDir, makeDirectory = FALSE)
+                    isDeletedFlag <- FALSE
+                }else break
+            }
+
+            #downloadedFiles <- getGEOSuppFiles(id, baseDir = destDir, makeDirectory = FALSE) %>% rownames()
 
             if(is.null(downloadedFiles)){
                 stop("Check the specified samples IDs to be valid. No file is found.")
@@ -104,9 +136,6 @@
 #' @param destDir The user path to save the downloaded files.
 #' @return A SummarizedExperiment object with appended expression values as assay and metadata as colData.
 #' @details This function is used internally by downloadGEO.
-#' @importFrom affy ReadAffy
-#' @importFrom affyPLM threestep
-#' @importFrom oligo read.celfiles rma
 #' @importFrom SummarizedExperiment SummarizedExperiment colData assay
 #' @importFrom dplyr %>%
 #' @importFrom Biobase exprs
@@ -123,23 +152,27 @@
         stop("The input metadata and sampleIDs do not match. Make sure the sample IDs match with geo_accession IDs from dataset.")
     }
 
-    #Normalize expression data based on RMA method
-    expression <- try({
-        ReadAffy(verbose = TRUE, celfile.path = destDir, sampleNames = sampleIDs, filenames = paste0(sampleIDs, '.CEL.gz')) %>%
-            threestep(background.method = "RMA.2", normalize.method = "quantile", summary.method = "median.polish") %>%
-            exprs() %>%
-            as.data.frame()
-    })
-
-    if ("try-error" %in% class(expression)) {
-        expression <- read.celfiles(file.path(destDir, paste0(sampleIDs, '.CEL.gz'))) %>%
-            rma(normalize = TRUE) %>%
-            exprs() %>%
-            as.data.frame()
-        if (sum(is.na(expression)) > 0) stop("There is NA in expression data.")
-        colnames(expression) <- sampleIDs
+    if (!.requirePackage("oligo")){
+        return(NULL)
     }
 
+    #Normalize expression data based on RMA method
+    # expression <- try({
+    #     ReadAffy(verbose = TRUE, celfile.path = destDir, sampleNames = sampleIDs, filenames = paste0(sampleIDs, '.CEL.gz')) %>%
+    #         threestep(background.method = "RMA.2", normalize.method = "quantile", summary.method = "median.polish") %>%
+    #         exprs() %>%
+    #         as.data.frame()
+    # })
+
+    # if ("try-error" %in% class(expression)) {
+    expression <- oligo::read.celfiles(file.path(destDir, paste0(sampleIDs, '.CEL.gz'))) %>%
+        oligo::rma(normalize = TRUE) %>%
+        Biobase::exprs() %>%
+        as.data.frame()
+    if (sum(is.na(expression)) > 0) stop("There is NA in expression data.")
+    colnames(expression) <- sampleIDs
+    # }
+
     if(dim(expression)[1] == 0 | dim(expression)[2] == 0){
         stop("The expression matrix is empty.")
     }
@@ -160,13 +193,14 @@
 #' @param metadata The metadat of downloaded GEO object.
 #' @param sampleIDs A vector of downloaded samples IDs from the queried GEO object.
 #' @param destDir The user path to save the downloaded files.
+#' @param greenOnly Logical, for use with source, should the green (Cy3) channel only be read, or are both red and green required.
 #' @return A SummarizedExperiment object with appended expression values as assay and metadata as colData.
 #' @details This function is used internally by downloadGEO.
 #' @importFrom SummarizedExperiment SummarizedExperiment colData assay
 #' @importFrom limma read.maimages backgroundCorrect normalizeWithinArrays normalizeBetweenArrays
 #' @importFrom dplyr %>%
 #' @noRd
-.processAgilent <- function(metadata, sampleIDs, destDir) {
+.processAgilent <- function(metadata, sampleIDs, destDir, greenOnly) {
 
     if (!dir.exists(destDir)) {
         stop("The destination directory does not exist.")
@@ -180,7 +214,7 @@
 
     raw.data <- read.maimages(file.path(destDir, paste0(sampleIDs, ".TXT.gz")),
                               source = "agilent",
-                              green.only = TRUE,
+                              green.only = greenOnly,
                               names = sampleIDs
     )
 
@@ -239,27 +273,22 @@
 #' @param platform The platform of selected GEO dataset.
 #' @param destDir A path to save downloaded data.
 #' @param protocol The protocol of the selected GEO dataset, including affymetrix and agilent.
+#' @param greenOnly Logical, for use with source, should the green (Cy3) channel only be read, or are both red and green required.
 #' @return A SummarizedExperiment object including the processed data.
 #' @examples
 #' \donttest{
 #' library(RCPA)
 #' # Affymetrix
-#' downloadPath <- file.path(tempdir(), "GSE5281")
+#' downloadPath <- file.path(tempdir(), "GSE230534")
 #' if(!dir.exists(downloadPath)) dir.create(downloadPath)
-#' affyDataset <- RCPA::downloadGEO(GEOID = "GSE5281", platform ="GPL570",
+#' affyDataset <- RCPA::downloadGEO(GEOID = "GSE230534", platform ="GPL23126",
 #'                                  protocol ="affymetrix", destDir = downloadPath)
-#'
-#' # Agilent
-#' downloadPath <- file.path(tempdir(), "GSE61196")
-#' if(!dir.exists(downloadPath)) dir.create(downloadPath)
-#' agilDataset <- RCPA::downloadGEO(GEOID = "GSE61196", platform ="GPL4133",
-#'                                  protocol ="agilent", destDir = downloadPath)
 #' }
 #' @importFrom SummarizedExperiment SummarizedExperiment colData assay
 #' @importFrom dplyr %>%
 #' @importFrom Biobase pData
 #' @export
-downloadGEO <- function(GEOID, platform, protocol = c("affymetrix", "agilent"), destDir) {
+downloadGEO <- function(GEOID, platform, protocol = c("affymetrix", "agilent"), destDir, greenOnly = TRUE) {
     protocol <- match.arg(protocol)
     protocol <- protocol %>% tolower()
 
@@ -280,15 +309,22 @@ downloadGEO <- function(GEOID, platform, protocol = c("affymetrix", "agilent"),
         stop("There is an error in downloading samples.")
     }
 
+    if(!is.logical(greenOnly)){
+        stop("The greenOnly parameter can only be TRUE or FALSE! Please refer to document.")
+    }
+
     if (protocol == "affymetrix"){
         #Normalize expression data for affymetrix using RMA method
         summarizedExperimentObject <- .processAffymetrix(GEOObject.metadata, sampleIDs, destDir)
     }else{
         #Normalize expression data for affymetrix using limma normexp, loess, and quantile methods
-        summarizedExperimentObject <- .processAgilent(GEOObject.metadata, sampleIDs, destDir)
+        summarizedExperimentObject <- .processAgilent(GEOObject.metadata, sampleIDs, destDir, greenOnly)
     }
 
     if(is.null(summarizedExperimentObject)){
+        if (pkgEnv$isMissingDependency){
+            return(NULL)
+        }
         stop("There is an error in processing and normalizing data.")
     }
 
